/*
 * WarriorDetachment.java
 *
 * Created on 18  2007 ., 15:20
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package universum.beings.Yeties;



import universum.bi.*;
import universum.util.*;
import java.util.List;
import java.util.ArrayList;

/**
 *
 * @author kalugin
 */
public class WarriorDetachment {
        
    
    public enum Status {
        INIT,
        NEED_MORE_POWER,
        PREPARE,
        SCOUTING,
        REQUEST_MORE_POWER,
        ATTACKING,
        WIN,
        LOSE
    }
    
    public enum Events {
        e0_enough_power,
        e1_everyone_assembled,
        e2_no_chance,
        e3_power_granted,
        e4_no_one_left,
        e5_victory,
        e6_not_scouted,
        no_events
    }
    

    private Status status;
    private Events event;
    
    private List<Yeti_2> detachment;
    
    private Yeti_2 scout;
    
    private Location assembly_point;
    public Location attack_point;
    
    private Field field;
    
    private int scout_turn;
    
    private int start_turn;
    
    private final int MAX_ASSEMBLY_TURNS = 50;
    
    private final float ATTACK_COEF_MASS = 2f;
    private final float ATTACK_COEF_ENERGY = 1.5f;
    
    private int enemy_power;
    
    private boolean force_attack;
    
    /** Creates a new instance of WarriorDetachment */
    public WarriorDetachment( Location attack, Field fld, BeingInterface bi ) {
        detachment = new ArrayList();
//        this.assembly_point = assembly;
        this.attack_point = attack;
        this.field = fld;
        this.assembly_point = field.selectAssemblyPoint( attack, bi );
        this.status = Status.INIT;
        this.event = Events.no_events;
        this.enemy_power = (int)field.getTotalEnemyMass( attack );
        this.start_turn = bi.getTurnsCount();
        this.scout_turn = 0;
        this.force_attack = false;
        this.enemy_power = 0;
    }
    
    public List<Yeti_2> getWarriorList() {
        return detachment;
    }
    
    public synchronized void nextTurn( BeingInterface bi ) {
        
        if( detachment.size() == 0 ) {
            System.out.println( "removing zombie detachment from state " + status.toString() + attack_point.toString() );
            status = Status.LOSE;
        }

        switch( status ) {
            case INIT:
                status = Status.NEED_MORE_POWER;
                break;
                
            case NEED_MORE_POWER:
                
                z0_check_my_power( bi );
                if( event == Events.e0_enough_power ) {
                    status = Status.PREPARE;
                }
                break;
                
            case PREPARE:
                z1_check_assembled( bi );
                if( event == Events.e1_everyone_assembled ) {
                    z2_send_scout( bi );
                    status = Status.SCOUTING;
                }
                break;
                
            case SCOUTING:
                z3_check_enemy_forces( bi );
                if( event == Events.e6_not_scouted ) {
                    break;
                }
                if( event == Events.e2_no_chance ) {
//                    status = Status.REQUEST_MORE_POWER;       !!!!!!!!!!!!!!!
//                    z4_start_attack( bi );
                    status = Status.NEED_MORE_POWER;
                    break;
                }
                if( event == Events.e0_enough_power ) {
                    status = Status.ATTACKING;
                    z4_start_attack( bi );
                    break;
                }
                break;
                
            case REQUEST_MORE_POWER:
                break;
                
            case ATTACKING:
//                System.out.println( "Attacking!!!!" + attack_point.toString() );
                z5_battle_control( bi );
                if( event == Events.e5_victory ) {
                    status = Status.WIN;
                    System.out.println( "We win!!!!" );
                    break;
                }
                if( event == Events.e4_no_one_left ) {
                    status = Status.LOSE;
                    System.out.println( "We loose!!!!" );
                    break;
                }
                break;
                
            case WIN:
                break;
                
            case LOSE:
                break;
        }
//        System.out.println( status.toString() + attack_point.toString() + " my_size=" + detachment.size() );
//        if( detachment.size() == 0 ) {
//            System.out.println( "I'm zombie!!!" + status.toString() + attack_point.toString() + " my_size=" + detachment.size() );
//        } else {
//            bi.log( detachment.get(0), status.toString() + attack_point.toString() + " my_size=" + detachment.size() );
//        }
    }
    
    public synchronized Status getCurrentStatus() {
        return status;
    }
    
    public void forceAttack( boolean flag ) {
        this.force_attack = flag;
    }
    
    public synchronized void z0_check_my_power( BeingInterface bi ) {
        float my_total_mass = 0;
        float my_total_energy = 0;
        for( Yeti_2 yeti : detachment ) {
            my_total_mass += yeti.M;
            my_total_energy += bi.getEnergy( yeti );
        }
        
        if( ( my_total_energy > ATTACK_COEF_ENERGY * enemy_power ) ||
                ( my_total_mass > ATTACK_COEF_MASS * enemy_power ) ||
              ( bi.getTurnsCount() > start_turn + MAX_ASSEMBLY_TURNS ) ||
                force_attack ) {
            event = Events.e0_enough_power;
        }
    }
    
    public synchronized void z1_check_assembled( BeingInterface bi ) {
        
        if ( bi.getTurnsCount() > start_turn + MAX_ASSEMBLY_TURNS ) {
            event = Events.e1_everyone_assembled;
            return;
        }
        
        for( Yeti_2 yeti : detachment ) {
            if( bi.getLocation( yeti ) != assembly_point ) {
                return;
            }
        }
        event = Events.e1_everyone_assembled;
    }
    
    public synchronized void z2_send_scout( BeingInterface bi ) {
        if( detachment.size() == 0 ) {
            event = Events.e2_no_chance;
            return;
        }
        
        Yeti_2 curr_scout = detachment.get( 0 );
        
        for( Yeti_2 yeti : detachment ) {
            if( yeti.M < curr_scout.M ) {
                curr_scout = yeti;
            }
        }
        
        curr_scout.getBeingKind().setAttackFlag( true );
        scout = curr_scout;
        
        scout_turn = bi.getTurnsCount();
    } 
    
    public synchronized void z3_check_enemy_forces( BeingInterface bi ) {
        
        if( bi.getTurnsCount() < scout_turn + 2 ) {
            event = Events.e6_not_scouted;
            return;
        }
        
        enemy_power = (int) field.getTotalEnemyMass( attack_point );
        
        float my_total_mass = 0;
        float my_total_energy = 0;
        
        for( Yeti_2 yeti : detachment ) {
            my_total_mass += yeti.M;
            my_total_energy += bi.getEnergy( yeti );
        }
        
        if( ( my_total_energy > enemy_power * 0.4 ) || ( my_total_mass > enemy_power ) ||
             ( bi.getTurnsCount() > start_turn + MAX_ASSEMBLY_TURNS ) || force_attack ) {
            event = Events.e0_enough_power;
        } else {
            event = Events.e2_no_chance;
        }
        
    }
    
    public synchronized void z4_start_attack( BeingInterface bi ) {
        for( Yeti_2 yeti : detachment ) {
            yeti.getBeingKind().setAttackFlag( true );
        }
    }
    
    public synchronized void z5_battle_control( BeingInterface bi ) {
        if( detachment.size() == 0 ) {
            event = Events.e4_no_one_left;
            return;
        }
        
        for( Yeti_2 yeti : detachment ) {
            if( yeti.getBeingKind().getVictoryFlag() ) {
                event = Events.e5_victory;
                yeti.getBeingKind().setVictoryFlag( false );
                return;
            }
        }
    }
    
    
    
    public synchronized void addNewWarrior( Yeti_2 yeti ) {
        BeingKind bk = new BeingKind();
        
        bk.switchTo( BeingKind.CurrentType.WARRIOR );
        
        bk.setAssemplyPoint( this.assembly_point );
        bk.setTargetPoint( this.attack_point );
        
        yeti.upgradeBeingKind( bk );
        
        detachment.add( yeti );
    }
    
    public synchronized void deleteWarrior( Yeti_2 yeti ) {
        detachment.remove( yeti );
    }
    
    
    public synchronized boolean isMyWarrior( Yeti_2 yeti ) {
        if( detachment.contains( yeti ) ) {
            return true;
        }
        return false;
    }
    
}
